* Column Generation Process
$eolcom //

$include header.inc

set is
    i(is)
    v
    isv(is,v)
    same(v,v)
    differ(v,v);

$gdxin data
$load v

$gdxin cols
$load is isv

option i<isv;

* Read branching info
$ifthen exist %nf%.gdx
$  gdxin %nf%
$  load same differ
$else
   same(v,v)   = no;
   differ(v,v) = no;
$endif

scalar acti; acti = card(i);

alias (v,w);

* Reduce i according to branching deciscion
loop(differ(v,w),
   i(i)$(isv(i,v) and isv(i,w)) = no);
loop(same(v,w),
   i(i)$(isv(i,v) xor isv(i,w)) = no);

variables y(is), z, s(v);
binary variables y;
positive variable s;

equations obj, defcover;

obj.. sum(i, y(i)) + sum(v, s(v)*card(v)) =e= z;

defcover(v).. sum(isv(i,v), y(i)) =g= 1 - s(v);

model master /all/;

* prevent reformuation
z.lo = -1; master.optfile=1;

scalar isfeasible /0/, isfractional /0/, lowerbnd /-inf/ , updincumbent /inf/;

set
   cols             column              / 1*%maxcolspm% /
   cc(cols)         new columns
   info             column info         / level, lower, upper /
parameters
   col_info(cols, info)
   defcover_c(cols,v) entries in defcover constraint
   numcolumns         number of columns /0/
   ypar(is)           fractional y variables;

execute_unload 'branch', same, differ;

solve master using rmip minimizing z;

ypar(i)$(y.l(i)<1-1e-6 and y.l(i)>1e-6) = y.l(i);

*alias (v,k)
*variable x(v,k), b(k);
*scalar color, value, rvalue;

if (master.modelstat <> 1,
  execute_unload '%minfo%.gdx', isfeasible, isfractional, lowerbnd, updincumbent, numcolumns;
else
  execute_load 'bchsol.gdx', col_info, defcover_c; option cc<col_info;
  abort$(card(cols)=card(cc)) 'increase maxcols';
  put fdebug 'New cols: ' card(cc):0:0;
  loop(cc,
     loop(is$(ord(is)=acti+1),
        isv(is,v) = defcover_c(cc,v);
        y.l(is)   = col_info(cc,'level');
        ypar(is)$(y.l(is)<1-1e-6 and y.l(is)>1e-6) = y.l(is));
     acti = acti+1;
     abort$(card(is)=acti) 'increase is';
     put / 'new col: '; loop(v$defcover_c(cc,v), put ' ' v.tl:0);
  );
  numcolumns = acti;
  execute_unload 'cols.gdx', is, isv;

  isfeasible                    = 1;
  isfractional                  = card(ypar);

  lowerbnd                      = z.l;
  updincumbent$(isfractional=0) = lowerbnd;

  put / 'in master: isfeas:' isfeasible:0:0 ' frac:' isfractional:0:0 ' lb:' lowerbnd
               ' updinc:' updincumbent ' numcols:' numcolumns:0:0 ;


$ontext
  color = 1; rvalue = 1; x.l(v,k) = 0; b.l(k) = 0;

* Michael's variable translation rule
  loop(is$y.l(is),
    b.l(k)$(ord(k)=color) = y.l(is);
    loop(isv(is,v),
       x.l(v,k)$(ord(k)=color) = y.l(is));
    color = color+1);

* Marco's variable translation rule (code below does not work properly)
  loop(is$y.l(is),
     value = min(rvalue, y.l(is));
     while (value>1e-6,
       loop(isv(is,v),
         x.l(v,k)$(ord(k)=color) = x.l(v,k) + value);
       b.l(k)$(ord(k)=color) = b.l(k) + value;
       rvalue = rvalue - value;
       value  = y.l(is) - value;
       acti = y.l(is);
       display color, value, rvalue, x.l, b.l, acti;
       if (rvalue<1e-6, color = color + 1; rvalue = 1; y.l(is) = y.l(is)-value)));
$offtext

* Solve the MIP in the root node to get an incumbent
$ifthen not exist %nf%.gdx
  if (isfractional,
    option i<isv;
    option optcr=0; master.optfile=0;
    solve master using mip minimizing z;
    updincumbent$(master.modelstat=1 or master.modelstat=8) = z.l
    put / '*** running mip';
    put$(master.modelstat=1 or master.modelstat=8) 'updating incumbent ' z.l /;
  );
$endif

  execute_unload '%minfo%.gdx', isfeasible, isfractional, lowerbnd,
               updincumbent, numcolumns, same, differ, ypar
*, x, b
;


);
