* Tree Manager
$eolcom //

$include header.inc // read data which may be changed by the user

Set  nodes            nodes                         /1*%nrnodes%/
     act(nodes)       active nodes (waiting list)
     cur(nodes)       current node
     newnodes(nodes)  nodes created by branching
     lastnode(nodes)  overall last node

Parameter
     lbbnd(nodes)     lower bound of active nodes
     lowerbnd         lower bound of current node   /-inf/
     globallobnd      global lower bound            /-inf/
     incumbent        best known integer soluation  /+inf/
     isfeasible       flag: solution is feasible (1) else 0   /0/
     isfractional     flag: solution is fractional (1) else 0 /0/
     updincumbent     objective value from integer solution for current node /+inf/
     nodecnt          node counter                  /0/
     numcolumns       number of columns             /0/
     tmpcnt, tmpcnt1  temporary counters
     agap             absolute gap
     gap              relative gap;

File flog log file  / bp.log /
     fx   file operation handle
Scalars
     starttime   current date (in days); starttime = jnow;

flog.ps=1; flog.ap=1; execute 'rm -f bp.log';
* initialize problem
execute '=gams %initfile%.gms lo=%gams.lo%'; abort$errorlevel 'Problems running %initfile%';

* initialization for search tree
act('1')      = yes;  // root node is first active node
lastnode('1') = yes;
lbbnd('1')    = -inf; // in the beginning lowerbound for first node is -inf
gap           = %rgap%+1;
agap          = %agap%+1;
while(card(act) and (jnow-starttime)*24*60*60<%exectime% and nodecnt<%nrnodes% and gap>%rgap% and agap>%agap%,
  put fdebug 'Node ' nodecnt:0:0 / 'act: '; loop(act(nodes), put nodes.tl:0 ' '); put /;
  put 'last: '; loop(lastnode(nodes), put nodes.tl:0 ' '); put /;

  // compute last node in waiting list
*  cur(nodes)=no; tmpcnt=1; loop(act, cur(act)$(tmpcnt=card(act)) = yes; tmpcnt=tmpcnt+1);
  cur(nodes)=no; tmpcnt1=1; tmpcnt=smin(act, lbbnd(act));
  loop(act$(lbbnd(act)=tmpcnt and tmpcnt1=1), cur(act) = yes; tmpcnt1=0);
  execute 'rm -f %minfo%.gdx';   // remove existing file
  loop(cur(nodes),
    putclose fdebug 'cur: ' nodes.tl:0;
    // call gams file with column generation process
    put_utility fx 'shell' / 'gams "%master%" --nf='nodes.tl:0' lo=%gams.lo%';
    abort$errorlevel 'Problems running %master%');
*    abort 'stop';

  // load data from node solution
  execute_load '%minfo%.gdx', isfeasible, isfractional, lowerbnd, updincumbent, numcolumns;
  putclose fdebug 'from master: isfeas:' isfeasible:0:0 ' frac:' isfractional:0:0 ' lb:' lowerbnd
             ' updinc:' updincumbent ' numcols:' numcolumns:0:0;
  tmpcnt1=0;
  if (isfeasible and updincumbent<incumbent, // better incumbent ?
    tmpcnt1=1;
    incumbent = updincumbent;                     // update best integer solution
    execute 'cp -f %minfo%.gdx %incumbent%.gdx'); // save incumbent

  newnodes(nodes) = no;
  if (isfeasible and isfractional and lowerbnd+1e-6<incumbent,
    execute_unload '%binfoin%.gdx', cur, lastnode;
    execute 'gams %branching% lo=%gams.lo%';        // call branching process
    abort$errorlevel 'Problems running %branching%'
    execute_load '%binfo%.gdx', newnodes, lastnode; // load information from branching
    lbbnd(newnodes) = lowerbnd;                     // set lower bound from parent for newnodes
    act(newnodes)   = yes;                          // take newnodes in waiting list
    put fdebug 'newnodes: ' loop(newnodes(nodes), put nodes.tl:0 ' '); put /;
  );

  act(cur)        = no;                             // remove current node from waiting list
  globallobnd     = incumbent;
  globallobnd$card(act) = smin(act, lbbnd(act));
  gap             = 1;
  gap$globallobnd = (incumbent-globallobnd)/globallobnd;
  agap            = inf;
  agap            = incumbent-globallobnd

* Logging
  put$(mod(nodecnt,10)=0) flog ' Node Act. Lev.   Objective IInf    Best Int.   Best Bound  Gap Cols' /;
  put flog;
  if (tmpcnt1, put$isfractional '+' ; put$(isfractional=0) '*'
  else put ' ');
  put nodecnt:4:0 ' ' card(act):4:0 ' '; put$(card(newnodes)=0) lowerbnd:12:4 'p' isfractional:4:0;
  put$card(newnodes) lowerbnd:12:4 ' ' isfractional:4:0;
  putclose ' ' incumbent:12:4 ' ' globallobnd:12:4 ' ' gap:4:2 ' ' numcolumns:4:0;

  put fdebug;
  if (tmpcnt1, put$isfractional '+' ; put$(isfractional=0) '*'
  else put ' ');
  put nodecnt:4:0 ' ' card(act):4:0 ' '; put$(card(newnodes)=0) lowerbnd:12:4 'p' isfractional:4:0;
  put$card(newnodes) lowerbnd:12:4 ' ' isfractional:4:0;
  putclose ' ' incumbent:12:4 ' ' globallobnd:12:4 ' ' gap:4:2 ' ' numcolumns:4:0;

  nodecnt         = nodecnt+1;

)
